;-----------------------------------------------------------------------------
; File:     hitachi.hd44780.inc
; Date:		01/30/2017
; Purpose:  HD44780 interface macros
;
; $Author: jcleland $
; $Log$
;-----------------------------------------------------------------------------

;Include once
#ifndef		__HITACHI_HD44780_INC
	#define	__HITACHI_HD44780_INC

;.............................................................................
;<editor-fold defaultstate="collapsed" desc="Preprocessor">
;
;Define ports that will be used to communicate with the display
#define  CTRL_PORT				PORTB
#define  CTRL_TRIS				TRISB
#define	DATA_PORT				PORTC
#define	DATA_TRIS				TRISC

;-----------------------------------------------------------------------------
;Define control port pins
#define	RS							4
#define	RW							5				;Read/Write pin. 0 is write, 1 read
#define	E							6

;-----------------------------------------------------------------------------
;Command definitions
#define	CLRDISP					b'00000001'	;RW=0, RS=0
#define  CURSHOME					b'00000010' ;     .
#define  ENTRYMODE				b'00000110' ;     .
#define  DISPON					b'00001000' ;     .
;</editor-fold>

;.............................................................................
;<editor-fold defaultstate="collapsed" desc=".hitachi.data ">
;Local data declaration
.hitachi.data        udata
lcdCount					res	.2
lcdDigit 				res	.3						;byte to digits registers

lcdTemp              res   .2
lcdByte					res	.1
lcdTest					res	.2

lcdTableLow				res	.1
lcdTableHigh			res	.1
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdInitDisplay ()">
; Macro:		_44780EnableDisplay
_lcdInitDisplay macro
		banksel		CTRL_PORT
		clrf			CTRL_PORT

		banksel		CTRL_TRIS
		bcf			CTRL_TRIS, RS
		bcf			CTRL_PORT, RW
		bcf			CTRL_PORT, E

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS
		bcf			CTRL_PORT, RW

	  _lcdDelay60ms

	  _lcdWriteCmd	b'00110000'					;init
	  _lcdDelay60ms								;Wait for at least 50ms

	  _lcdWriteCmd	b'00110000'					;init
	  _lcdDelay60ms								;Wait for at least 100us

	  _lcdWriteCmd	b'00110000'					;init
	  _lcdDelay60ms								;Wait for at least 100us

	  _lcdWriteCmd	b'00111000'					;8-bit interface, 2 lines, 5x7 font
	  _lcdDelay60ms

	  _lcdWriteCmd	b'00001000'					;Display off, no cursor, no blink
  	   call			lcdBusyWait

	  _lcdWriteCmd	b'00000001'					;Clear display
  	   call			lcdBusyWait

	  _lcdWriteCmd	b'00000110'					;Cursor inc right, no display shift
  	   call			lcdBusyWait

 	  _lcdWriteCmd	b'00001100'					;Display on, no cursor, no blink
  	   call			lcdBusyWait
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_strobeE ()">
; Macro:		_strobeE
_lcdStrobeE macro
  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bsf			CTRL_PORT, E
	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdHomeCursor ()">
; Macro:		_lcdHomeCursor
_lcdHomeCursor macro
  	   call			lcdBusyWait

		banksel		DATA_PORT
		movlw			b'00000010'
		movwf			DATA_PORT

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS
		bcf			CTRL_PORT, RW
		bsf			CTRL_PORT, E
	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdSetCursor (pos)">
; Macro:		_lcdSetCursor
_lcdSetCursor macro _pos
		banksel		lcdTemp+4
		movlw			_pos
		addlw			0x80
		movwf		   lcdTemp+4

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS
		bcf			CTRL_PORT, RW
		bsf			CTRL_PORT, E

		banksel		lcdTemp+4
		movf			lcdTemp+4, w
		banksel		DATA_PORT
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteChar (bits)">
; Macro:		_lcdWriteCharW
_lcdWriteChar macro bits
  	   call			lcdBusyWait

		movlw			bits
		banksel		lcdTemp
		movwf			lcdTemp

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bsf			CTRL_PORT, RS				;Set data value
	   bcf			CTRL_PORT, RW				;Set write
		bsf			CTRL_PORT, E

		banksel		lcdTemp
		movf			lcdTemp, w
		banksel		DATA_PORT
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteCharW ()">
; Macro:		_lcdWriteCharW
_lcdWriteCharW macro
		banksel		lcdTemp
		movwf			lcdTemp

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bsf			CTRL_PORT, RS				;Set data value
	   bcf			CTRL_PORT, RW				;Set write
		bsf			CTRL_PORT, E

		banksel		lcdTemp
		movf			lcdTemp, w
		banksel		DATA_PORT
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteCmd (bits)">
; Macro:		_lcdWriteCharW
_lcdWriteCmd macro bits
		movlw			bits
		banksel		lcdTemp
		movwf			lcdTemp

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS				;Set data value
	   bcf			CTRL_PORT, RW				;Set write
		bsf			CTRL_PORT, E

		banksel		lcdTemp
		movf			lcdTemp, w
		banksel		DATA_PORT
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteCmdW ()">
; Macro:		_lcdWriteCharW
_lcdWriteCmdW macro
		banksel		lcdTemp
		movwf			lcdTemp

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS				;Set data value
	   bcf			CTRL_PORT, RW				;Set write
		bsf			CTRL_PORT, E

		banksel		lcdTemp
		movf			lcdTemp, w
		banksel		DATA_PORT
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdByteToInt (byte, digits)">
; Function:    _lcdByteToInt
; Call Depth:  0
; Purpose:     Converts an 8-byte value into a 3 digit number, the digits
;              stored in digit<0:2> variables.
_lcdByteToInt macro _byte, _digits
	local ___lcdPositive, ___lcdByteToInt_STARTUP,___lcdByteToInt_HUNDREDS,
	local ___lcdByteToInt_TENS, ___lcdByteToInt_ONES
		movf			_byte, w
		movwf			lcdByte

      ;Test for two's complement
      btfss       lcdByte, 7
         goto     ___lcdPositive

      comf        lcdByte, f
      incf        lcdByte, f

___lcdPositive
		bankisel		_digits
		banksel		_digits
		movlw			_digits
		movwf			FSR

___lcdByteToInt_100
      movlw       .100							;Move a decimal 100 into the work register
      incf        INDF, f						;Increment the 100's digit

      banksel     lcdByte
      subwf       lcdByte, f              ;Subtract the byte value from 100 and
      btfsc       STATUS,C						; jump on carry if we unborrowed
         goto        ___lcdByteToInt_100 ;No? go again

      banksel     lcdByte
      addwf       lcdByte,f               ; put the value back in the temp reg
      decf        INDF, f						;Take the last 100's inc away and
      incf        FSR, f						;Move to next digit
___lcdByteToInt_10
      movlw       .10
      incf        INDF,f

      banksel     lcdByte
      subwf       lcdByte,f
      btfsc       STATUS,C
         goto     ___lcdByteToInt_10

      banksel     lcdByte
      addwf       lcdByte,f
      decf        INDF,f
      incf        FSR, f
___lcdByteToInt_1
      banksel     lcdByte
      movf        lcdByte, w
      movwf       INDF
  endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteInt (buffer, num)">
; Macro:			_lcdWriteDigits
; Arguments:	buffer - A buffer containing integer values to be printed
;					num - The number of digits (bytes) contained within the buffer
;
;					This macro expects the digits to be in the order that they would
;					be displayed. That is to say, left to right. Leading zeros
;					should be ignored and replaced with spaces...
_lcdWriteInt macro _digitBuffer, _numDigits
		banksel		_numDigits
      movlw       _numDigits					; save lvalue for byte count
      banksel     lcdCount						;Select count bank and
      movwf       lcdCount						;Move to file register

      bankisel    _digitBuffer	   		;Indirect bank select for buffer
      movlw       _digitBuffer   			;Copy buffer address through W
      movwf       FSR							; into FSR

___writeDigitsLoop
      movf        INDF, w						;Move first digit into accumulator
      incf        FSR, f						;Increment FSR to next digit
		addlw			0x30							; and add ASCII offset

     _lcdWriteCharW								;Write accumulator digit to LCD

      banksel     lcdCount						;Bank select for counter
      decfsz      lcdCount, f					;Decrement the counter and
         goto     ___writeDigitsLoop		; continue until all digits written
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdWriteTable (table)">
; Macro:			_lcdWriteTable
_lcdWriteTable macro table
      banksel     lcdTableHigh				;Select bank
      movlw       HIGH table					;Get the high bits of the table addr
      movwf       lcdTableHigh				;Store them in our file register
      movlw       LOW table					;Get the low bits
      movwf       lcdTableLow					;And store those
      call        lcdWriteTable		;Call our function to send the table data
   endm
;</editor-fold>

;.............................................................................
;                                  Delays

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdDelay160us ()">
; Macro:				_LCDLogDelay160us
; Purpose:        Sleeps for 160us. This is used for LCD initialization
_lcdDelay160us macro
	local	___LCDLogD160us
      banksel     _sleep_time_us+2         ;Select bank
      movlw       .78                  ;Load loop count
      movwf       _sleep_time_us+2         ; into temporary register
___LCDLogD160us
      nop                              ;Waste 4 cycles per loop
      decfsz      _sleep_time_us+2,f       ;Decrement temp register
			goto     ___LCDLogD160us       ;Loop
      nop                              ;Delay=2+2+2+(38*4-1)+1+2=320=160us
   endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdDelay60us ()">
; Macro:		      _LCDLogDelay60us
; Purpose:        Sleeps for 60us. This is used for LCD initialization
_lcdDelay60us	macro
	local ___LCDLogD60us
      banksel     _sleep_time_us+2         ;Select bank
      movlw       .28                  ;Load loop count
      movwf       _sleep_time_us+2         ; into temporary register
___LCDLogD60us
      nop                              ;Waste 4 cycles per loop
      decfsz      _sleep_time_us+2,f       ;Decrement temp register
       goto       ___LCDLogD60us        ;Loop
      nop                              ;Delay=2+2+2+(28*4-1)+1+2=120=60us
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdDelay5ms ()">
; Function:       _LCDLogDelay5ms
; Purpose:        Sleeps for 5ms. This is used for LCD initialization. This
;                 delay is not exact, but it doesn't need to be.
_lcdDelay5ms macro
	local ___LCDLogD5ms, ___LCDLogD5ms2
      banksel     _sleep_time_us+2         ;Select bank
      movlw       .8                   ;Load loop count
      movwf       _sleep_time_us+1         ; store in temp register
___LCDLogD5ms
      movlw       .249                 ;Load loop count into accumulator
      movwf       _sleep_time_us+2         ; and store in temp register
___LCDLogD5ms2
      nop
      nop
      decfsz      _sleep_time_us+2,f
       goto       ___LCDLogD5ms2        ;Inner loop is 5*250-1
      decfsz      _sleep_time_us+1,f
       goto       ___LCDLogD5ms         ;Outer loop is 8*(5+5*249-1)-1
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdDelay20ms ()">
; Function:       _LCDLogDelay20ms
; Purpose:        Sleeps for 20ms. This is used for LCD initialization. Like
;                 the 5ms delay, this is not an exact delay, but precision is
;                 not important.
_lcdDelay20ms macro
	local ___LCDLogD20ms, ___LCDLogD20ms2
      banksel     _sleep_time_us+2         ;Select bank
      movlw       .32                  ;Load loop count
      movwf       _sleep_time_us+1         ; store in temp register
___LCDLogD20ms
      movlw       .249                 ;Load loop count into accumulator
      movwf       _sleep_time_us+2         ; and store in temp register
___LCDLogD20ms2
      nop
      nop
      decfsz      _sleep_time_us+2,f
       goto       ___LCDLogD20ms2       ;Inner loop is 5*250-1
      decfsz      _sleep_time_us+1,f
       goto       ___LCDLogD20ms        ;Outer loop is 32*(5+5*249-1)-1
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_lcdDelay60ms ()">
; Function:       _LCDLogDelay60ms
; Purpose:        Sleeps for 60ms. This is used for LCD initialization. Like
;                 the 5ms delay, this is not an exact delay, but precision is
;                 not important.
_lcdDelay60ms macro
	  _lcdDelay20ms
	  _lcdDelay20ms
	  _lcdDelay20ms
	endm
;</editor-fold>
;............................................................................
;                                 Functions

.hitachi.code		code

;=============================================================================
;<editor-fold defaultstate="collapsed" desc="lcdWriteTable">
; Function:    lcdWriteTable
; Purpose:     This function sends the contents of the table specified in the
;              file registers LOG_tablelo and LOG_tablehi to the log serial
;              pin. This function is normally called from the LOG_table
;              macro which sets up the PCLATH and PCL register data. It also
;              pre-increments the low word address data and increments the
;              hi bit file register if the data crosses a page boundry. This
;              is done by determining when the low bits wrap on increment and
;              set the carry flag.
lcdWriteTable
      banksel     lcdTableHigh				;Select bank
__LCDLogWriteTable_start
      call        lcdTableLookup				;Jump to lookup function
      addlw       0								;See if the accumulator result is 0,
      btfsc       STATUS,Z						; indicating EOL
			goto       __LCDLogWriteTable_end;Accumulator is 0, exit
     _lcdWriteCharW	   		;Write the accumulator value and loop
      goto        __LCDLogWriteTable_start;Loop
__LCDLogWriteTable_end
   return
   global lcdWriteTable
;</editor-fold>

;=============================================================================
;<editor-fold defaultstate="collapsed" desc="lcdTableLookup">
; Function:    __LCDLogTableLookup
; Purpose:     This private function is called by the __LOG_table function
;              to perform the table hop and accumulator load. It relies on
;              the LOG_tablelo and LOG_tablehi variables set to the table
;              start address. This function returns the present address
;              value and increments the file registers for the next read. It
;              will also allow tables that cross page boundries as it checks
;              the low order address byte for wrapping after increment and
;              bumps the PCLATH-holding file register as required.
;-----------------------------------------------------------------------------
lcdTableLookup ;Internal subroutine, assumes bank selected correctly
      movf        lcdTableHigh,w				;Load hi bits to accumulator
      movwf       PCLATH						;put this in the PCLATH register
      movf        lcdTableLow,w				;Load the low bits to accumulator
      incf        lcdTableLow,f				;Increment low data register for next
      btfsc       STATUS,Z						;Skip the hi bank inc if no zero flag
      incf        lcdTableHigh,f				;The next low addr crosses a page,
      movwf       PCL							;Jump to table
														;Return is implicit from table jump
;</editor-fold>

;=============================================================================
;<editor-fold defaultstate="collapsed" desc="lcdWriteCharW ">
; Function:		_lcdWriteCharW
lcdWriteCharW
		;save W
		banksel		lcdTemp
		movwf			lcdTemp

  	   call			lcdBusyWait

		banksel		CTRL_PORT
		bcf			CTRL_PORT, RS				;Set data value
	   bcf			CTRL_PORT, RW				;Set write
		bsf			CTRL_PORT, E

		banksel		DATA_PORT
		movf			lcdTemp, w
		movwf			DATA_PORT

	  _Delay2us
		bcf			CTRL_PORT, E
	return
;</editor-fold>

;=============================================================================
;<editor-fold defaultstate="collapsed" desc="lcdBusyWait ">
; Function:		_lcdWriteCharW
; Purpose:		This function blocks until the LCD is no longer in a busy state
lcdBusyWait
		banksel	DATA_TRIS
		movlw		b'11111111'						;Set PORTB for input
		movwf		DATA_TRIS

		banksel	CTRL_PORT
		bcf		CTRL_PORT, RS					; Set LCD for command mode
		bsf		CTRL_PORT, RW					; Setup to read busy flag
		bsf		CTRL_PORT, E					; LCD E-line High
		MOVF		DATA_PORT, w					; Read busy flag + DDram address
		bcf		CTRL_PORT, E					; LCD E-line Low
		andlw		0x80								; Check Busy flag, High = Busy
		btfss		STATUS, Z						;Check if flag set
			goto	lcdBusyWait						; continue waiting

___lcdBusyWait_idle
		banksel	CTRL_PORT
		bcf		CTRL_PORT, RW
		bsf		STATUS, RP0						; Select Register page 1

		MOVLW		b'00000000'
		banksel  DATA_TRIS
		movwf		DATA_TRIS						; Set PORTB for output
	  _lcdDelay160us
	return
;</editor-fold>
#endif ;Include once