
;-----------------------------------------------------------------------------
; File:     util.inc
; Date:		01/30/2017
; Purpose:  Contains utility functions for delays, etc
;
; $Author: jcleland $
;-----------------------------------------------------------------------------

;Include once
#ifndef		__UTIL_INC
	#define  __UTIL_INC

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc=".util.data">
;Local data declaration
;
.util.data        udata
_sleep_time_us    res   .3                ;General use register
utilTemp			res	.4
utilLoop			res	.2
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="__TuneOscillator (port, pin)">
; Macro:    __TuneOscillator
; Purpose:  This macro produces a pulse on the specified pin at 4 instruction
;				cycles, useful for tuning the internal oscillator.
__TuneOscillator macro port, pin
	local __label1
		banksel		port
__label1
		bsf			port, pin
		nop
		nop
		nop
		bcf			port, pin
		nop
		goto			__label1
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_byteOut (buffer, port, pinSignal, pinClock)">
; Macro:    _byteOut
; Purpose:  Writes bits out for the buffer specified on the
_byteOut macro _byte, _port, _pinA, _pinB
	local ___byteOut_loop,___byteOut_set,___byteOut_clear,___byteOut_bitdone
		banksel		utilLoop					;Bank select for loop counter
		movlw			.8								;Accumulator loaded with counter
		movwf			utilLoop					;Move to loop variable

		banksel		_byte
		movf			_byte, w
		banksel		utilTemp+3
		movwf			utilTemp+3


		banksel		_port
		bcf			_port, _pinA
___byteOut_loop
		banksel		utilTemp+3
		rrf			utilTemp+3, f					;Rotate through carry
		btfss			STATUS, C					; and check to see if this bit is set
			goto		___byteOut_clear
		goto			___byteOut_set

___byteOut_set
		banksel		_port
		bsf			_port, _pinA
	  _Delay2us

		banksel		_port
		bsf			_port, _pinB
		goto			___byteOut_bitdone

___byteOut_clear
		banksel		_port
		bcf			_port, _pinA
	  _Delay2us

		banksel		_port
		bsf			_port, _pinB

___byteOut_bitdone
	  _Delay4us
		banksel		_port
		bcf			_port, _pinB
		banksel		utilLoop
   	decfsz		utilLoop, f
			goto     ___byteOut_loop

		banksel		_port
		bcf			_port, _pinA
	endm
;</editor-fold>

;-----------------------------------------------------------------------------
;<editor-fold defaultstate="collapsed" desc="_memset (reg, value, bytes)">
; Macro:       _memset(reg, value, bytes)
; Purpose:     This function is used to set a range of contiguous memory to
;              a specific value. 'reg' is considered the pointer to the
;              head of the buffer and 'bytes' is the number of bytes to set.
;              all data will be initialized with 'value'.
;-----------------------------------------------------------------------------
_memset macro _reg, _value, _bytes
   local ___memset_loop
      movlw       _reg							;Load accumulator with register address
      movwf       FSR							;Store in indirect register

      bankisel    _reg							;Set indirect address bit STATUS,IRP
      movlw       _bytes						;Move the literal byte count to accumulator

      banksel     utilTemp+4					;Select counter register
      movwf       utilTemp+4					;Store byte count
      movlw       _value						;Load the value
___memset_loop
      movwf       INDF							;Initialize memory
      incf        FSR,f							;Increment pointer register
      decfsz      utilTemp+4,f				;Decrement counter and loop?
			goto       ___memset_loop			; still counting, keep looping...
   endm												;Done.
;</editor-fold>

;-----------------------------------------------------------------------------
; Macro:    _Delay1us
; Purpose:  Wastes 1us of time
; Accuracy: Dead-on
_Delay1us macro
   local __label1
      goto __label1
__label1
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay2us
; Purpose:  Wastes 2us of time
; Accuracy: Dead-on
_Delay2us macro
   local __label1, __label2, __label3
		nop
      goto __label1
__label1
		nop
      goto __label2
__label2
      goto __label3
__label3
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay4us
; Purpose:  Wastes 4us of time
; Accuracy: Dead-on
_Delay4us macro
	local __loop
		movlw			.2
      movwf       _sleep_time_us    ; store in counter register, 2 cycles
__loop
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ; each loop takes 3 cycles, last one is 2
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay10us
; Purpose:  Wastes 10us of time
; Accuracy: Dead-on (40 cycles)
;-----------------------------------------------------------------------------
_Delay10us macro
   local __loop
      movlw       .4                ;Load accumulator for 5 loops
      movwf       _sleep_time_us    ; store in counter register, 2 cycles
__loop
		nop
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ;
		nop
   endm                             ;


;-----------------------------------------------------------------------------
; Macro:    _Delay60us
; Purpose:  Wastes 60us of time
; Accuracy: Dead-on
;-----------------------------------------------------------------------------
_Delay60us macro
   local __loop
      banksel     _sleep_time_us    ;Select bank (2 cycles)
      movlw       .29               ;Load accumulator with loop count
      movwf       _sleep_time_us    ; store in counter register (2 cycles)
__loop
		nop
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ;
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay100us
; Purpose:  Wastes 100us of time
; Accuracy: Dead-on
;-----------------------------------------------------------------------------
_Delay100us macro
   local __loop
      banksel     _sleep_time_us    ;Select bank (2 cycles)
      movlw       .65               ;Load accumulator for 65 loops
      movwf       _sleep_time_us    ; store in counter register, 2 cycles
__loop
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ; each loop takes 3 cycles, last one is 2
      nop                           ;Extra cycle
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay250us
; Purpose:  Wastes 250us of time
; Accuracy: Dead-on
;-----------------------------------------------------------------------------
_Delay250us macro
   local __loop
      banksel     _sleep_time_us    ;Select bank (2 cycles)
      movlw       .165              ;Load accumulator for 165 loops
      movwf       _sleep_time_us    ; store in counter register, 2 cycles
__loop
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ; each loop takes 3 cycles, last one is 2
      nop                           ;Extra cycle
   endm

;-----------------------------------------------------------------------------
; Macro:    _Delay500us
; Purpose:  Wastes 500us of time
; Accuracy: Dead-on
;-----------------------------------------------------------------------------
_Delay500us macro
   local __loop
      banksel     _sleep_time_us    ;Select bank (2 cycles)
      movlw       .249              ;Load accumulator for 249 loops
      movwf       _sleep_time_us    ; store in counter register, 2 cycles
__loop
      nop                           ;Waste cycle
      decfsz      _sleep_time_us,f  ;Decrement the counter and loop
       goto       __loop            ; each loop takes 3 cycles, last one is 2
   endm

;Include once
#endif